@@ -18,7 +18,7 @@ pub trait IndexList<T> {
 }
 
 // TODO: remove traits here and make this const fn new
-/// IndexedBucket works like a bucket but has a secondary index
+/// `IndexedMap` works like a `Map` but has a secondary index
 pub struct IndexedMap<'a, K, T, I>
 where
     K: PrimaryKey<'a>,
@@ -129,23 +129,13 @@ where
         self.primary.may_load(store, key)
     }
 
-    // use prefix to scan -> range
-    pub fn prefix(&self, p: K::Prefix) -> Prefix<Vec<u8>, T> {
-        Prefix::new(self.pk_namespace, &p.prefix())
-    }
-
-    // use sub_prefix to scan -> range
-    pub fn sub_prefix(&self, p: K::SubPrefix) -> Prefix<Vec<u8>, T> {
-        Prefix::new(self.pk_namespace, &p.prefix())
-    }
-
     // use no_prefix to scan -> range
-    fn no_prefix(&self) -> Prefix<Vec<u8>, T> {
+    fn no_prefix_raw(&self) -> Prefix<Vec<u8>, T> {
         Prefix::new(self.pk_namespace, &[])
     }
 }
 
-// short-cut for simple keys, rather than .prefix(()).range(...)
+// short-cut for simple keys, rather than .prefix(()).range_raw(...)
 impl<'a, K, T, I> IndexedMap<'a, K, T, I>
 where
     K: PrimaryKey<'a>,
@@ -154,7 +144,7 @@ where
 {
     // I would prefer not to copy code from Prefix, but no other way
     // with lifetimes (create Prefix inside function and return ref = no no)
-    pub fn range<'c>(
+    pub fn range_raw<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<Bound>,
@@ -164,7 +154,17 @@ where
     where
         T: 'c,
     {
-        self.no_prefix().range(store, min, max, order)
+        self.no_prefix_raw().range_raw(store, min, max, order)
+    }
+
+    pub fn keys_raw<'c>(
+        &self,
+        store: &'c dyn Storage,
+        min: Option<Bound>,
+        max: Option<Bound>,
+        order: cosmwasm_std::Order,
+    ) -> Box<dyn Iterator<Item = Vec<u8>> + 'c> {
+        self.no_prefix_raw().keys_raw(store, min, max, order)
     }
 }
 
@@ -175,11 +175,12 @@ where
     T: Serialize + DeserializeOwned + Clone,
     I: IndexList<T>,
 {
-    /// while range assumes you set the prefix to one element and call range over the last one,
-    /// prefix_range accepts bounds for the lowest and highest elements of the Prefix we wish to
-    /// accept, and iterates over those. There are some issues that distinguish these to and blindly
-    /// casting to Vec<u8> doesn't solve them.
-    pub fn prefix_range<'c>(
+    /// While `range_raw` over a `prefix` fixes the prefix to one element and iterates over the
+    /// remaining, `prefix_range_raw` accepts bounds for the lowest and highest elements of the `Prefix`
+    /// itself, and iterates over those (inclusively or exclusively, depending on `PrefixBound`).
+    /// There are some issues that distinguish these two, and blindly casting to `Vec<u8>` doesn't
+    /// solve them.
+    pub fn prefix_range_raw<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<PrefixBound<'a, K::Prefix>>,
@@ -196,6 +197,22 @@ where
     }
 }
 
+#[cfg(feature = "iterator")]
+impl<'a, K, T, I> IndexedMap<'a, K, T, I>
+where
+    T: Serialize + DeserializeOwned + Clone,
+    K: PrimaryKey<'a>,
+    I: IndexList<T>,
+{
+    pub fn sub_prefix(&self, p: K::SubPrefix) -> Prefix<K::SuperSuffix, T> {
+        Prefix::new(self.pk_namespace, &p.prefix())
+    }
+
+    pub fn prefix(&self, p: K::Prefix) -> Prefix<K::Suffix, T> {
+        Prefix::new(self.pk_namespace, &p.prefix())
+    }
+}
+
 #[cfg(feature = "iterator")]
 impl<'a, K, T, I> IndexedMap<'a, K, T, I>
 where
@@ -203,11 +220,13 @@ where
     K: PrimaryKey<'a> + KeyDeserialize,
     I: IndexList<T>,
 {
-    /// while range_de assumes you set the prefix to one element and call range over the last one,
-    /// prefix_range_de accepts bounds for the lowest and highest elements of the Prefix we wish to
-    /// accept, and iterates over those. There are some issues that distinguish these to and blindly
-    /// casting to Vec<u8> doesn't solve them.
-    pub fn prefix_range_de<'c>(
+    /// While `range` over a `prefix` fixes the prefix to one element and iterates over the
+    /// remaining, `prefix_range` accepts bounds for the lowest and highest elements of the
+    /// `Prefix` itself, and iterates over those (inclusively or exclusively, depending on
+    /// `PrefixBound`).
+    /// There are some issues that distinguish these two, and blindly casting to `Vec<u8>` doesn't
+    /// solve them.
+    pub fn prefix_range<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<PrefixBound<'a, K::Prefix>>,
@@ -220,12 +239,12 @@ where
         K: 'c,
         K::Output: 'static,
     {
-        let mapped = namespaced_prefix_range(store, self.primary.namespace(), min, max, order)
+        let mapped = namespaced_prefix_range(store, self.pk_namespace, min, max, order)
             .map(deserialize_kv::<K, T>);
         Box::new(mapped)
     }
 
-    pub fn range_de<'c>(
+    pub fn range<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<Bound>,
@@ -236,10 +255,10 @@ where
         T: 'c,
         K::Output: 'static,
     {
-        self.no_prefix_de().range_de(store, min, max, order)
+        self.no_prefix().range(store, min, max, order)
     }
 
-    pub fn keys_de<'c>(
+    pub fn keys<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<Bound>,
@@ -250,11 +269,11 @@ where
         T: 'c,
         K::Output: 'static,
     {
-        self.no_prefix_de().keys_de(store, min, max, order)
+        self.no_prefix().keys(store, min, max, order)
     }
 
-    fn no_prefix_de(&self) -> Prefix<K, T> {
-        Prefix::new(self.primary.namespace(), &[])
+    fn no_prefix(&self) -> Prefix<K, T> {
+        Prefix::new(self.pk_namespace, &[])
     }
 }
 
@@ -262,8 +281,8 @@ where
 mod test {
     use super::*;
 
-    use crate::indexes::{index_string_tuple, index_triple, MultiIndex, UniqueIndex};
-    use crate::U32Key;
+    use crate::indexes::index_string_tuple;
+    use crate::{index_tuple, MultiIndex, UniqueIndex};
     use cosmwasm_std::testing::MockStorage;
     use cosmwasm_std::{MemoryStorage, Order};
     use serde::{Deserialize, Serialize};
@@ -276,10 +295,10 @@ mod test {
     }
 
     struct DataIndexes<'a> {
-        // Second arg is for storing pk
-        pub name: MultiIndex<'a, (Vec<u8>, Vec<u8>), Data>,
-        pub age: UniqueIndex<'a, U32Key, Data>,
-        pub name_lastname: UniqueIndex<'a, (Vec<u8>, Vec<u8>), Data>,
+        // Last type parameters are for signaling pk deserialization
+        pub name: MultiIndex<'a, String, Data, String>,
+        pub age: UniqueIndex<'a, u32, Data, String>,
+        pub name_lastname: UniqueIndex<'a, (Vec<u8>, Vec<u8>), Data, String>,
     }
 
     // Future Note: this can likely be macro-derived
@@ -292,8 +311,8 @@ mod test {
 
     // For composite multi index tests
     struct DataCompositeMultiIndex<'a> {
-        // Third arg needed for storing pk
-        pub name_age: MultiIndex<'a, (Vec<u8>, U32Key, Vec<u8>), Data>,
+        // Last type parameter is for signaling pk deserialization
+        pub name_age: MultiIndex<'a, (Vec<u8>, u32), Data, String>,
     }
 
     // Future Note: this can likely be macro-derived
@@ -307,8 +326,8 @@ mod test {
     // Can we make it easier to define this? (less wordy generic)
     fn build_map<'a>() -> IndexedMap<'a, &'a str, Data, DataIndexes<'a>> {
         let indexes = DataIndexes {
-            name: MultiIndex::new(|d, k| (d.name.as_bytes().to_vec(), k), "data", "data__name"),
-            age: UniqueIndex::new(|d| U32Key::new(d.age), "data__age"),
+            name: MultiIndex::new(|d| d.name.clone(), "data", "data__name"),
+            age: UniqueIndex::new(|d| d.age, "data__age"),
             name_lastname: UniqueIndex::new(
                 |d| index_string_tuple(&d.name, &d.last_name),
                 "data__name_lastname",
@@ -395,20 +414,17 @@ mod test {
         let count = map
             .idx
             .name
-            .prefix(b"Maria".to_vec())
-            .range(&store, None, None, Order::Ascending)
+            .prefix("Maria".to_string())
+            .range_raw(&store, None, None, Order::Ascending)
             .count();
         assert_eq!(2, count);
 
-        // TODO: we load by wrong keys - get full storage key!
-
-        // load it by secondary index (we must know how to compute this)
-        // let marias: Vec<_>> = map
+        // load it by secondary index
         let marias: Vec<_> = map
             .idx
             .name
-            .prefix(b"Maria".to_vec())
-            .range(&store, None, None, Order::Ascending)
+            .prefix("Maria".to_string())
+            .range_raw(&store, None, None, Order::Ascending)
             .collect::<StdResult<_>>()
             .unwrap();
         assert_eq!(2, marias.len());
@@ -420,8 +436,8 @@ mod test {
         let count = map
             .idx
             .name
-            .prefix(b"Marib".to_vec())
-            .range(&store, None, None, Order::Ascending)
+            .prefix("Marib".to_string())
+            .range_raw(&store, None, None, Order::Ascending)
             .count();
         assert_eq!(0, count);
 
@@ -429,8 +445,8 @@ mod test {
         let count = map
             .idx
             .name
-            .prefix(b"Mari`".to_vec())
-            .range(&store, None, None, Order::Ascending)
+            .prefix("Mari`".to_string())
+            .range_raw(&store, None, None, Order::Ascending)
             .count();
         assert_eq!(0, count);
 
@@ -438,50 +454,50 @@ mod test {
         let count = map
             .idx
             .name
-            .prefix(b"Maria5".to_vec())
-            .range(&store, None, None, Order::Ascending)
+            .prefix("Maria5".to_string())
+            .range_raw(&store, None, None, Order::Ascending)
             .count();
         assert_eq!(0, count);
 
         // index_key() over MultiIndex works (empty pk)
         // In a MultiIndex, an index key is composed by the index and the primary key.
         // Primary key may be empty (so that to iterate over all elements that match just the index)
-        let key = (b"Maria".to_vec(), b"".to_vec());
-        // Use the index_key() helper to build the (raw) index key
+        let key = "Maria".to_string();
+        // Use the index_key() helper to build the (raw) index key with an empty pk
         let key = map.idx.name.index_key(key);
         // Iterate using a bound over the raw key
         let count = map
             .idx
             .name
-            .range(&store, Some(Bound::inclusive(key)), None, Order::Ascending)
+            .range_raw(&store, Some(Bound::inclusive(key)), None, Order::Ascending)
             .count();
         // gets from the first "Maria" until the end
         assert_eq!(4, count);
 
         // index_key() over MultiIndex works (non-empty pk)
         // Build key including a non-empty pk
-        let key = (b"Maria".to_vec(), b"1".to_vec());
-        // Use the index_key() helper to build the (raw) index key
-        let key = map.idx.name.index_key(key);
+        let key = ("Maria".to_string(), "1".to_string());
+        // Use the joined_key() helper to build the (raw) index key
+        let key = key.joined_key();
         // Iterate using a (exclusive) bound over the raw key.
         // (Useful for pagination / continuation contexts).
         let count = map
             .idx
             .name
-            .range(&store, Some(Bound::exclusive(key)), None, Order::Ascending)
+            .range_raw(&store, Some(Bound::exclusive(key)), None, Order::Ascending)
             .count();
         // gets from the 2nd "Maria" until the end
         assert_eq!(3, count);
 
         // index_key() over UniqueIndex works.
-        let age_key = U32Key::from(23);
+        let age_key = 23u32;
         // Use the index_key() helper to build the (raw) index key
         let age_key = map.idx.age.index_key(age_key);
         // Iterate using a (inclusive) bound over the raw key.
         let count = map
             .idx
             .age
-            .range(
+            .range_raw(
                 &store,
                 Some(Bound::inclusive(age_key)),
                 None,
@@ -492,17 +508,73 @@ mod test {
         assert_eq!(4, count);
 
         // match on proper age
-        let proper = U32Key::new(42);
+        let proper = 42u32;
         let aged = map.idx.age.item(&store, proper).unwrap().unwrap();
         assert_eq!(pk, String::from_vec(aged.0).unwrap());
         assert_eq!(*data, aged.1);
 
         // no match on wrong age
-        let too_old = U32Key::new(43);
+        let too_old = 43u32;
         let aged = map.idx.age.item(&store, too_old).unwrap();
         assert_eq!(None, aged);
     }
 
+    #[test]
+    fn range_raw_simple_key_by_multi_index() {
+        let mut store = MockStorage::new();
+        let map = build_map();
+
+        // save data
+        let data1 = Data {
+            name: "Maria".to_string(),
+            last_name: "".to_string(),
+            age: 42,
+        };
+        let pk = "5627";
+        map.save(&mut store, pk, &data1).unwrap();
+
+        let data2 = Data {
+            name: "Juan".to_string(),
+            last_name: "Perez".to_string(),
+            age: 13,
+        };
+        let pk = "5628";
+        map.save(&mut store, pk, &data2).unwrap();
+
+        let data3 = Data {
+            name: "Maria".to_string(),
+            last_name: "Williams".to_string(),
+            age: 24,
+        };
+        let pk = "5629";
+        map.save(&mut store, pk, &data3).unwrap();
+
+        let data4 = Data {
+            name: "Maria Luisa".to_string(),
+            last_name: "Bemberg".to_string(),
+            age: 12,
+        };
+        let pk = "5630";
+        map.save(&mut store, pk, &data4).unwrap();
+
+        let marias: Vec<_> = map
+            .idx
+            .name
+            .prefix("Maria".to_string())
+            .range_raw(&store, None, None, Order::Descending)
+            .collect::<StdResult<_>>()
+            .unwrap();
+        let count = marias.len();
+        assert_eq!(2, count);
+
+        // Pks, sorted by (descending) pk
+        assert_eq!(marias[0].0, b"5629");
+        assert_eq!(marias[1].0, b"5627");
+        // Data is correct
+        assert_eq!(marias[0].1, data3);
+        assert_eq!(marias[1].1, data1);
+    }
+
     #[test]
     fn range_simple_key_by_multi_index() {
         let mut store = MockStorage::new();
@@ -544,31 +616,27 @@ mod test {
         let marias: Vec<_> = map
             .idx
             .name
-            .prefix(b"Maria".to_vec())
+            .prefix("Maria".to_string())
             .range(&store, None, None, Order::Descending)
             .collect::<StdResult<_>>()
             .unwrap();
         let count = marias.len();
         assert_eq!(2, count);
 
-        // Sorted by (descending) pk
-        assert_eq!(marias[0].0, b"5629");
-        assert_eq!(marias[1].0, b"5627");
+        // Pks, sorted by (descending) pk
+        assert_eq!(marias[0].0, "5629");
+        assert_eq!(marias[1].0, "5627");
         // Data is correct
         assert_eq!(marias[0].1, data3);
         assert_eq!(marias[1].1, data1);
     }
 
     #[test]
-    fn range_composite_key_by_multi_index() {
+    fn range_raw_composite_key_by_multi_index() {
         let mut store = MockStorage::new();
 
         let indexes = DataCompositeMultiIndex {
-            name_age: MultiIndex::new(
-                |d, k| index_triple(&d.name, d.age, k),
-                "data",
-                "data__name_age",
-            ),
+            name_age: MultiIndex::new(|d| index_tuple(&d.name, d.age), "data", "data__name_age"),
         };
         let map = IndexedMap::new("data", indexes);
 
@@ -605,6 +673,67 @@ mod test {
         let pk4: &[u8] = b"5630";
         map.save(&mut store, pk4, &data4).unwrap();
 
+        let marias: Vec<_> = map
+            .idx
+            .name_age
+            .sub_prefix(b"Maria".to_vec())
+            .range_raw(&store, None, None, Order::Descending)
+            .collect::<StdResult<_>>()
+            .unwrap();
+        let count = marias.len();
+        assert_eq!(2, count);
+
+        // Pks, sorted by (descending) age
+        assert_eq!(pk1, marias[0].0);
+        assert_eq!(pk3, marias[1].0);
+
+        // Data
+        assert_eq!(data1, marias[0].1);
+        assert_eq!(data3, marias[1].1);
+    }
+
+    #[test]
+    fn range_composite_key_by_multi_index() {
+        let mut store = MockStorage::new();
+
+        let indexes = DataCompositeMultiIndex {
+            name_age: MultiIndex::new(|d| index_tuple(&d.name, d.age), "data", "data__name_age"),
+        };
+        let map = IndexedMap::new("data", indexes);
+
+        // save data
+        let data1 = Data {
+            name: "Maria".to_string(),
+            last_name: "".to_string(),
+            age: 42,
+        };
+        let pk1 = "5627";
+        map.save(&mut store, pk1, &data1).unwrap();
+
+        let data2 = Data {
+            name: "Juan".to_string(),
+            last_name: "Perez".to_string(),
+            age: 13,
+        };
+        let pk2 = "5628";
+        map.save(&mut store, pk2, &data2).unwrap();
+
+        let data3 = Data {
+            name: "Maria".to_string(),
+            last_name: "Young".to_string(),
+            age: 24,
+        };
+        let pk3 = "5629";
+        map.save(&mut store, pk3, &data3).unwrap();
+
+        let data4 = Data {
+            name: "Maria Luisa".to_string(),
+            last_name: "Bemberg".to_string(),
+            age: 43,
+        };
+        let pk4 = "5630";
+        map.save(&mut store, pk4, &data4).unwrap();
+
         let marias: Vec<_> = map
             .idx
             .name_age
@@ -615,7 +744,7 @@ mod test {
         let count = marias.len();
         assert_eq!(2, count);
 
-        // Pks (sorted by age descending)
+        // Pks, sorted by (descending) age
         assert_eq!(pk1, marias[0].0);
         assert_eq!(pk3, marias[1].0);
 
@@ -645,14 +774,14 @@ mod test {
 
         // query by unique key
         // match on proper age
-        let age42 = U32Key::new(42);
-        let (k, v) = map.idx.age.item(&store, age42.clone()).unwrap().unwrap();
+        let age42 = 42u32;
+        let (k, v) = map.idx.age.item(&store, age42).unwrap().unwrap();
         assert_eq!(String::from_vec(k).unwrap(), pks[0]);
         assert_eq!(v.name, datas[0].name);
         assert_eq!(v.age, datas[0].age);
 
         // match on other age
-        let age23 = U32Key::new(23);
+        let age23 = 23u32;
         let (k, v) = map.idx.age.item(&store, age23).unwrap().unwrap();
         assert_eq!(String::from_vec(k).unwrap(), pks[1]);
         assert_eq!(v.name, datas[1].name);
@@ -698,8 +827,8 @@ mod test {
          -> usize {
             map.idx
                 .name
-                .prefix(name.as_bytes().to_vec())
-                .keys(store, None, None, Order::Ascending)
+                .prefix(name.to_string())
+                .keys_raw(store, None, None, Order::Ascending)
                 .count()
         };
 
@@ -732,7 +861,40 @@ mod test {
     }
 
     #[test]
-    fn unique_index_simple_key_range() {
+    fn range_raw_simple_key_by_unique_index() {
+        let mut store = MockStorage::new();
+        let map = build_map();
+
+        // save data
+        let (pks, datas) = save_data(&mut store, &map);
+
+        let res: StdResult<Vec<_>> = map
+            .idx
+            .age
+            .range_raw(&store, None, None, Order::Ascending)
+            .collect();
+        let ages = res.unwrap();
+
+        let count = ages.len();
+        assert_eq!(5, count);
+
+        // The pks, sorted by age ascending
+        assert_eq!(pks[3], String::from_slice(&ages[0].0).unwrap()); // 12
+        assert_eq!(pks[1], String::from_slice(&ages[1].0).unwrap()); // 23
+        assert_eq!(pks[2], String::from_slice(&ages[2].0).unwrap()); // 32
+        assert_eq!(pks[0], String::from_slice(&ages[3].0).unwrap()); // 42
+        assert_eq!(pks[4], String::from_slice(&ages[4].0).unwrap()); // 90
+
+        // The associated data
+        assert_eq!(datas[3], ages[0].1);
+        assert_eq!(datas[1], ages[1].1);
+        assert_eq!(datas[2], ages[2].1);
+        assert_eq!(datas[0], ages[3].1);
+        assert_eq!(datas[4], ages[4].1);
+    }
+
+    #[test]
+    fn range_simple_key_by_unique_index() {
         let mut store = MockStorage::new();
         let map = build_map();
 
@@ -750,22 +912,51 @@ mod test {
         assert_eq!(5, count);
 
         // The pks, sorted by age ascending
-        assert_eq!(pks[4], String::from_slice(&ages[4].0).unwrap());
-        assert_eq!(pks[3], String::from_slice(&ages[0].0).unwrap());
-        assert_eq!(pks[1], String::from_slice(&ages[1].0).unwrap());
-        assert_eq!(pks[2], String::from_slice(&ages[2].0).unwrap());
-        assert_eq!(pks[0], String::from_slice(&ages[3].0).unwrap());
+        assert_eq!(pks[3], ages[0].0);
+        assert_eq!(pks[1], ages[1].0);
+        assert_eq!(pks[2], ages[2].0);
+        assert_eq!(pks[0], ages[3].0);
+        assert_eq!(pks[4], ages[4].0);
 
         // The associated data
-        assert_eq!(datas[4], ages[4].1);
         assert_eq!(datas[3], ages[0].1);
         assert_eq!(datas[1], ages[1].1);
         assert_eq!(datas[2], ages[2].1);
         assert_eq!(datas[0], ages[3].1);
+        assert_eq!(datas[4], ages[4].1);
     }
 
     #[test]
-    fn unique_index_composite_key_range() {
+    fn range_raw_composite_key_by_unique_index() {
+        let mut store = MockStorage::new();
+        let map = build_map();
+
+        // save data
+        let (pks, datas) = save_data(&mut store, &map);
+
+        let res: StdResult<Vec<_>> = map
+            .idx
+            .name_lastname
+            .prefix(b"Maria".to_vec())
+            .range_raw(&store, None, None, Order::Ascending)
+            .collect();
+        let marias = res.unwrap();
+
+        // Only two people are called "Maria"
+        let count = marias.len();
+        assert_eq!(2, count);
+
+        // The pks
+        assert_eq!(pks[0], String::from_slice(&marias[0].0).unwrap());
+        assert_eq!(pks[1], String::from_slice(&marias[1].0).unwrap());
+
+        // The associated data
+        assert_eq!(datas[0], marias[0].1);
+        assert_eq!(datas[1], marias[1].1);
+    }
+
+    #[test]
+    fn range_composite_key_by_unique_index() {
         let mut store = MockStorage::new();
         let map = build_map();
 
@@ -785,8 +976,8 @@ mod test {
         assert_eq!(2, count);
 
         // The pks
-        assert_eq!(pks[0], String::from_slice(&marias[0].0).unwrap());
-        assert_eq!(pks[1], String::from_slice(&marias[1].0).unwrap());
+        assert_eq!(pks[0], marias[0].0);
+        assert_eq!(pks[1], marias[1].0);
 
         // The associated data
         assert_eq!(datas[0], marias[0].1);
@@ -795,7 +986,7 @@ mod test {
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn range_de_simple_string_key() {
+    fn range_simple_string_key() {
         let mut store = MockStorage::new();
         let map = build_map();
 
@@ -803,7 +994,7 @@ mod test {
         let (pks, datas) = save_data(&mut store, &map);
 
         // let's try to iterate!
-        let all: StdResult<Vec<_>> = map.range_de(&store, None, None, Order::Ascending).collect();
+        let all: StdResult<Vec<_>> = map.range(&store, None, None, Order::Ascending).collect();
         let all = all.unwrap();
         assert_eq!(
             all,
@@ -816,7 +1007,7 @@ mod test {
 
         // let's try to iterate over a range
         let all: StdResult<Vec<_>> = map
-            .range_de(
+            .range(
                 &store,
                 Some(Bound::Inclusive(b"3".to_vec())),
                 None,
@@ -838,15 +1029,38 @@ mod test {
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn prefix_range_de() {
+    fn prefix_simple_string_key() {
+        let mut store = MockStorage::new();
+        let map = build_map();
+
+        // save data
+        let (pks, datas) = save_data(&mut store, &map);
+
+        // Let's prefix and iterate.
+        // This is similar to calling range() directly, but added here for completeness / prefix
+        // type checks
+        let all: StdResult<Vec<_>> = map
+            .prefix(())
+            .range(&store, None, None, Order::Ascending)
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(
+            all,
+            pks.clone()
+                .into_iter()
+                .map(str::to_string)
+                .zip(datas.into_iter())
+                .collect::<Vec<_>>()
+        );
+    }
+
+    #[test]
+    #[cfg(feature = "iterator")]
+    fn prefix_composite_key() {
         let mut store = MockStorage::new();
 
         let indexes = DataCompositeMultiIndex {
-            name_age: MultiIndex::new(
-                |d, k| index_triple(&d.name, d.age, k),
-                "data",
-                "data__name_age",
-            ),
+            name_age: MultiIndex::new(|d| index_tuple(&d.name, d.age), "data", "data__name_age"),
         };
         let map = IndexedMap::new("data", indexes);
 
@@ -856,7 +1070,7 @@ mod test {
             last_name: "".to_string(),
             age: 42,
         };
-        let pk1: (&[u8], &[u8]) = (b"1", b"5627");
+        let pk1 = ("1", "5627");
         map.save(&mut store, pk1, &data1).unwrap();
 
         let data2 = Data {
@@ -864,7 +1078,7 @@ mod test {
             last_name: "Perez".to_string(),
             age: 13,
         };
-        let pk2: (&[u8], &[u8]) = (b"2", b"5628");
+        let pk2 = ("2", "5628");
         map.save(&mut store, pk2, &data2).unwrap();
 
         let data3 = Data {
@@ -872,7 +1086,7 @@ mod test {
             last_name: "Young".to_string(),
             age: 24,
         };
-        let pk3: (&[u8], &[u8]) = (b"2", b"5629");
+        let pk3 = ("2", "5629");
         map.save(&mut store, pk3, &data3).unwrap();
 
         let data4 = Data {
@@ -880,58 +1094,138 @@ mod test {
             last_name: "Bemberg".to_string(),
             age: 43,
         };
-        let pk4: (&[u8], &[u8]) = (b"3", b"5630");
+        let pk4 = ("3", "5630");
         map.save(&mut store, pk4, &data4).unwrap();
 
-        // let's try to iterate!
+        // let's prefix and iterate
         let result: StdResult<Vec<_>> = map
-            .prefix_range_de(
-                &store,
-                Some(PrefixBound::inclusive(&b"2"[..])),
-                None,
-                Order::Ascending,
-            )
+            .prefix("2")
+            .range(&store, None, None, Order::Ascending)
             .collect();
         let result = result.unwrap();
         assert_eq!(
             result,
-            [
-                ((b"2".to_vec(), b"5628".to_vec()), data2.clone()),
-                ((b"2".to_vec(), b"5629".to_vec()), data3.clone()),
-                ((b"3".to_vec(), b"5630".to_vec()), data4)
-            ]
+            [("5628".to_string(), data2), ("5629".to_string(), data3),]
         );
+    }
 
-        // let's try to iterate over a range
+    #[test]
+    #[cfg(feature = "iterator")]
+    fn prefix_triple_key() {
+        let mut store = MockStorage::new();
+
+        let indexes = DataCompositeMultiIndex {
+            name_age: MultiIndex::new(|d| index_tuple(&d.name, d.age), "data", "data__name_age"),
+        };
+        let map = IndexedMap::new("data", indexes);
+
+        // save data
+        let data1 = Data {
+            name: "Maria".to_string(),
+            last_name: "".to_string(),
+            age: 42,
+        };
+        let pk1 = ("1", "1", "5627");
+        map.save(&mut store, pk1, &data1).unwrap();
+
+        let data2 = Data {
+            name: "Juan".to_string(),
+            last_name: "Perez".to_string(),
+            age: 13,
+        };
+        let pk2 = ("1", "2", "5628");
+        map.save(&mut store, pk2, &data2).unwrap();
+
+        let data3 = Data {
+            name: "Maria".to_string(),
+            last_name: "Young".to_string(),
+            age: 24,
+        };
+        let pk3 = ("2", "1", "5629");
+        map.save(&mut store, pk3, &data3).unwrap();
+
+        let data4 = Data {
+            name: "Maria Luisa".to_string(),
+            last_name: "Bemberg".to_string(),
+            age: 43,
+        };
+        let pk4 = ("2", "2", "5630");
+        map.save(&mut store, pk4, &data4).unwrap();
+
+        // let's prefix and iterate
         let result: StdResult<Vec<_>> = map
-            .prefix_range_de(
-                &store,
-                Some(PrefixBound::inclusive(&b"2"[..])),
-                Some(PrefixBound::exclusive(&b"3"[..])),
-                Order::Ascending,
-            )
+            .prefix(("1", "2"))
+            .range(&store, None, None, Order::Ascending)
+            .collect();
+        let result = result.unwrap();
+        assert_eq!(result, [("5628".to_string(), data2),]);
+    }
+
+    #[test]
+    #[cfg(feature = "iterator")]
+    fn sub_prefix_triple_key() {
+        let mut store = MockStorage::new();
+
+        let indexes = DataCompositeMultiIndex {
+            name_age: MultiIndex::new(|d| index_tuple(&d.name, d.age), "data", "data__name_age"),
+        };
+        let map = IndexedMap::new("data", indexes);
+
+        // save data
+        let data1 = Data {
+            name: "Maria".to_string(),
+            last_name: "".to_string(),
+            age: 42,
+        };
+        let pk1 = ("1", "1", "5627");
+        map.save(&mut store, pk1, &data1).unwrap();
+
+        let data2 = Data {
+            name: "Juan".to_string(),
+            last_name: "Perez".to_string(),
+            age: 13,
+        };
+        let pk2 = ("1", "2", "5628");
+        map.save(&mut store, pk2, &data2).unwrap();
+
+        let data3 = Data {
+            name: "Maria".to_string(),
+            last_name: "Young".to_string(),
+            age: 24,
+        };
+        let pk3 = ("2", "1", "5629");
+        map.save(&mut store, pk3, &data3).unwrap();
+
+        let data4 = Data {
+            name: "Maria Luisa".to_string(),
+            last_name: "Bemberg".to_string(),
+            age: 43,
+        };
+        let pk4 = ("2", "2", "5630");
+        map.save(&mut store, pk4, &data4).unwrap();
+
+        // let's sub-prefix and iterate
+        let result: StdResult<Vec<_>> = map
+            .sub_prefix("1")
+            .range(&store, None, None, Order::Ascending)
             .collect();
         let result = result.unwrap();
         assert_eq!(
             result,
             [
-                ((b"2".to_vec(), b"5628".to_vec()), data2),
-                ((b"2".to_vec(), b"5629".to_vec()), data3),
+                (("1".to_string(), "5627".to_string()), data1),
+                (("2".to_string(), "5628".to_string()), data2),
             ]
         );
     }
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn prefix_range_de_triple_key() {
+    fn prefix_range_simple_key() {
         let mut store = MockStorage::new();
 
         let indexes = DataCompositeMultiIndex {
-            name_age: MultiIndex::new(
-                |d, k| index_triple(&d.name, d.age, k),
-                "data",
-                "data__name_age",
-            ),
+            name_age: MultiIndex::new(|d| index_tuple(&d.name, d.age), "data", "data__name_age"),
         };
         let map = IndexedMap::new("data", indexes);
 
@@ -941,7 +1235,7 @@ mod test {
             last_name: "".to_string(),
             age: 42,
         };
-        let pk1: (&[u8], &[u8], &[u8]) = (b"1", b"1", b"5627");
+        let pk1 = ("1", "5627");
         map.save(&mut store, pk1, &data1).unwrap();
 
         let data2 = Data {
@@ -949,7 +1243,7 @@ mod test {
             last_name: "Perez".to_string(),
             age: 13,
         };
-        let pk2: (&[u8], &[u8], &[u8]) = (b"1", b"2", b"5628");
+        let pk2 = ("2", "5628");
         map.save(&mut store, pk2, &data2).unwrap();
 
         let data3 = Data {
@@ -957,7 +1251,7 @@ mod test {
             last_name: "Young".to_string(),
             age: 24,
         };
-        let pk3: (&[u8], &[u8], &[u8]) = (b"2", b"1", b"5629");
+        let pk3 = ("2", "5629");
         map.save(&mut store, pk3, &data3).unwrap();
 
         let data4 = Data {
@@ -965,14 +1259,95 @@ mod test {
             last_name: "Bemberg".to_string(),
             age: 43,
         };
-        let pk4: (&[u8], &[u8], &[u8]) = (b"2", b"2", b"5630");
+        let pk4 = ("3", "5630");
         map.save(&mut store, pk4, &data4).unwrap();
 
-        // let's try to iterate!
+        // let's prefix-range and iterate
         let result: StdResult<Vec<_>> = map
-            .prefix_range_de(
+            .prefix_range(
                 &store,
-                Some(PrefixBound::inclusive((&b"1"[..], &b"2"[..]))),
+                Some(PrefixBound::inclusive("2")),
+                None,
+                Order::Ascending,
+            )
+            .collect();
+        let result = result.unwrap();
+        assert_eq!(
+            result,
+            [
+                (("2".to_string(), "5628".to_string()), data2.clone()),
+                (("2".to_string(), "5629".to_string()), data3.clone()),
+                (("3".to_string(), "5630".to_string()), data4)
+            ]
+        );
+
+        // let's try to iterate over a more restrictive prefix-range!
+        let result: StdResult<Vec<_>> = map
+            .prefix_range(
+                &store,
+                Some(PrefixBound::inclusive("2")),
+                Some(PrefixBound::exclusive("3")),
+                Order::Ascending,
+            )
+            .collect();
+        let result = result.unwrap();
+        assert_eq!(
+            result,
+            [
+                (("2".to_string(), "5628".to_string()), data2),
+                (("2".to_string(), "5629".to_string()), data3),
+            ]
+        );
+    }
+
+    #[test]
+    #[cfg(feature = "iterator")]
+    fn prefix_range_triple_key() {
+        let mut store = MockStorage::new();
+
+        let indexes = DataCompositeMultiIndex {
+            name_age: MultiIndex::new(|d| index_tuple(&d.name, d.age), "data", "data__name_age"),
+        };
+        let map = IndexedMap::new("data", indexes);
+
+        // save data
+        let data1 = Data {
+            name: "Maria".to_string(),
+            last_name: "".to_string(),
+            age: 42,
+        };
+        let pk1 = ("1", "1", "5627");
+        map.save(&mut store, pk1, &data1).unwrap();
+
+        let data2 = Data {
+            name: "Juan".to_string(),
+            last_name: "Perez".to_string(),
+            age: 13,
+        };
+        let pk2 = ("1", "2", "5628");
+        map.save(&mut store, pk2, &data2).unwrap();
+
+        let data3 = Data {
+            name: "Maria".to_string(),
+            last_name: "Young".to_string(),
+            age: 24,
+        };
+        let pk3 = ("2", "1", "5629");
+        map.save(&mut store, pk3, &data3).unwrap();
+
+        let data4 = Data {
+            name: "Maria Luisa".to_string(),
+            last_name: "Bemberg".to_string(),
+            age: 43,
+        };
+        let pk4 = ("2", "2", "5630");
+        map.save(&mut store, pk4, &data4).unwrap();
+
+        // let's prefix-range and iterate
+        let result: StdResult<Vec<_>> = map
+            .prefix_range(
+                &store,
+                Some(PrefixBound::inclusive(("1", "2"))),
                 None,
                 Order::Ascending,
             )
@@ -982,23 +1357,26 @@ mod test {
             result,
             [
                 (
-                    (b"1".to_vec(), b"2".to_vec(), b"5628".to_vec()),
+                    ("1".to_string(), "2".to_string(), "5628".to_string()),
                     data2.clone()
                 ),
                 (
-                    (b"2".to_vec(), b"1".to_vec(), b"5629".to_vec()),
+                    ("2".to_string(), "1".to_string(), "5629".to_string()),
                     data3.clone()
                 ),
-                ((b"2".to_vec(), b"2".to_vec(), b"5630".to_vec()), data4)
+                (
+                    ("2".to_string(), "2".to_string(), "5630".to_string()),
+                    data4
+                )
             ]
         );
 
-        // let's try to iterate over a range
+        // let's prefix-range over inclusive bounds on both sides
         let result: StdResult<Vec<_>> = map
-            .prefix_range_de(
+            .prefix_range(
                 &store,
-                Some(PrefixBound::inclusive((&b"1"[..], &b"2"[..]))),
-                Some(PrefixBound::inclusive((&b"2"[..], &b"1"[..]))),
+                Some(PrefixBound::inclusive(("1", "2"))),
+                Some(PrefixBound::inclusive(("2", "1"))),
                 Order::Ascending,
             )
             .collect();
@@ -1006,18 +1384,23 @@ mod test {
         assert_eq!(
             result,
             [
-                ((b"1".to_vec(), b"2".to_vec(), b"5628".to_vec()), data2),
-                ((b"2".to_vec(), b"1".to_vec(), b"5629".to_vec()), data3),
+                (
+                    ("1".to_string(), "2".to_string(), "5628".to_string()),
+                    data2
+                ),
+                (
+                    ("2".to_string(), "1".to_string(), "5629".to_string()),
+                    data3
+                ),
             ]
         );
     }
 
     mod inclusive_bound {
         use super::*;
-        use crate::U64Key;
 
         struct Indexes<'a> {
-            secondary: MultiIndex<'a, (U64Key, Vec<u8>), u64>,
+            secondary: MultiIndex<'a, u64, u64>,
         }
 
         impl<'a> IndexList<u64> for Indexes<'a> {
@@ -1032,7 +1415,7 @@ mod test {
         fn composite_key_query() {
             let indexes = Indexes {
                 secondary: MultiIndex::new(
-                    |secondary, k| (U64Key::new(*secondary), k),
+                    |secondary| *secondary,
                     "test_map",
                     "test_map__secondary",
                 ),
@@ -1046,10 +1429,10 @@ mod test {
             let items: Vec<_> = map
                 .idx
                 .secondary
-                .prefix_range(
+                .prefix_range_raw(
                     &store,
                     None,
-                    Some(PrefixBound::inclusive(1)),
+                    Some(PrefixBound::inclusive(1u64)),
                     Order::Ascending,
                 )
                 .collect::<Result<_, _>>()
